// dbx-min-coverage: 0% because dialog
/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */

package com.dropbox.android.activity.payment

import android.content.Context
import android.support.design.widget.BottomSheetBehavior
import android.support.design.widget.BottomSheetDialog
import android.widget.Button
import android.widget.FrameLayout
import android.widget.TextView
import com.dropbox.android.R
import com.dropbox.android.util.spannedFromHtml
import com.dropbox.core.ui.util.ViewUtil
import com.dropbox.android.util.tintDrawable
import java.text.DateFormat

class PaymentMethodPicker(context: Context) : BottomSheetDialog(context) {

    companion object {
        /**
         * One month in milliseconds
         */
        private const val ONE_MONTH_MS = 1000 * 60 * 60 * 24 * 30L
    }

    private val bodyText by lazy {
        checkNotNull(findViewById<TextView>(R.id.body_text))
    }
    private val googlePlayButton by lazy {
        checkNotNull(findViewById<Button>(R.id.button_use_google_play))
    }
    private val useCCButton by lazy {
        checkNotNull(findViewById<Button>(R.id.button_enter_credit_card))
    }
    private val bottomSheet by lazy {
        checkNotNull(findViewById<FrameLayout>(R.id.design_bottom_sheet))
    }

    init {
        setContentView(R.layout.payment_method_picker)

        ViewUtil.runBeforeNextDraw(bottomSheet) {
            // Disable peeking behavior
            BottomSheetBehavior.from(bottomSheet).peekHeight = bottomSheet.height
            false
        }

        val trialExpirationTime = System.currentTimeMillis() - ONE_MONTH_MS

        val playIcon = tintDrawable(
            context, R.drawable.ic_google_play_black_24dp, R.color.dbx_maestro_stone)
        val ccIcon = tintDrawable(
            context, R.drawable.outline_credit_card_black_24, R.color.dbx_maestro_stone)

        googlePlayButton.setCompoundDrawablesWithIntrinsicBounds(playIcon, null, null, null)
        useCCButton.setCompoundDrawablesWithIntrinsicBounds(ccIcon, null, null, null)

        bodyText.text = spannedFromHtml(context.getString(
            R.string.payment_method_picker_body,
            DateFormat.getDateInstance(DateFormat.LONG).format(trialExpirationTime)))
    }

    fun setOnGooglePlaySelectedListener(listener: () -> Unit) {
        googlePlayButton.setOnClickListener {
            listener()
            dismiss()
        }
    }

    fun setOnCCSelectedListener(listener: () -> Unit) {
        useCCButton.setOnClickListener {
            listener()
            dismiss()
        }
    }
}
