// dbx-min-coverage: 0% because wiring
/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */

package com.dropbox.android.analytics

import android.content.Context
import com.dropbox.android.exception.EnvInfo
import com.dropbox.android.settings.GlobalProperties
import com.dropbox.android.user.DbxUserManager
import com.dropbox.android.user.DbxUserset
import com.dropbox.core.android.analytics_impl.AnalyticsApi
import com.dropbox.core.android.analytics_impl.AnalyticsInitializationDependenciesModule
import com.dropbox.core.android.analytics_impl.AnalyticsExceptionReporter
import com.dropbox.core.android.analytics_impl.UsersIdsProvider
import com.dropbox.core.crash_reporting.CrashLogger
import com.dropbox.hairball.util.NetworkManager
import com.dropbox.internalclient.NoAuthApi
import java.io.File
import java.io.InputStream

fun newAnalyticsDependenciesModule(
    globalProperties: GlobalProperties,
    dbxUserManager: DbxUserManager,
    envInfo: EnvInfo,
    networkManager: NetworkManager,
    noAuthApi: NoAuthApi,
    crashLogger: CrashLogger,
    context: Context
): AnalyticsInitializationDependenciesModule {
    val usersIdsProvider = object : UsersIdsProvider {
        override val userIds: Set<String>
            get() = DbxUserset.getUserIds(dbxUserManager.users)
        override val userIdsAsString: String?
            get() = dbxUserManager.users?.id

        override fun addOnUsersIdsChangedBackgroundThreadListener(
            listener: UsersIdsProvider.Listener
        ) {
            dbxUserManager.registerOnUsersetChangedBackgroundThreadListener {
                listener.onUserIdsChangedBackgroundThread(DbxUserset.getUserIds(it))
            }
        }
    }

    val analyticsApi = object : AnalyticsApi {
        override fun sendLog(
            appVersion: String,
            userIds: Collection<String>,
            deviceId: String,
            sysModel: String,
            sysVersion: String,
            ts: Long,
            logData: InputStream,
            streamSize: Long,
            gzipped: Boolean
        ) {
            noAuthApi.sendLog(
                    appVersion,
                    NoAuthApi.LogLevel.ANALYTICS,
                    userIds, deviceId,
                    sysModel,
                    sysVersion,
                    ts,
                    logData,
                    streamSize,
                    gzipped
            )
        }
    }

    val analyticsExceptionReporter: AnalyticsExceptionReporter =
            object : AnalyticsExceptionReporter {
                override fun report(exception: Exception) {
                    crashLogger.outputWarn(exception)
                }
    }

    return AnalyticsInitializationDependenciesModule(
            globalProperties,
            usersIdsProvider,
            envInfo,
            networkManager,
            analyticsApi,
            analyticsExceptionReporter,
            File(context.filesDir, "anl"))
}
