/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */

package com.dropbox.android.banner

import android.view.View
import com.dropbox.android.activity.base.BaseActivity
import com.dropbox.android.browser.banners.BannerHeaderItem
import com.dropbox.android.browser.banners.CameraUploadsAdminDisabeldControlBannerHeaderItem
import com.dropbox.android.user.DbxUser
import com.dropbox.android.user.DbxUserset
import com.dropbox.android.util.ActivitiesHelper
import com.dropbox.android.util.web.WebUrl
import com.dropbox.base.analytics.CameraUploadsBannersEvents

open class CameraUploadsAdminDisabledControlBanner(
    private val activitiesHelper: ActivitiesHelper
) :
        BaseBanner<CameraUploadsAdminDisabeldControlBannerHeaderItem>() {

    override fun bannerActionShouldDismissBanner(): Boolean {
        return false
    }

    override fun shouldShowBanner(
        bannerContext: BannerContext,
        userset: DbxUserset
    ): Boolean {
        val user = bannerContext.user ?: return false
        val cameraUploadsManager = bannerContext.cameraUploadsManager

        return cameraUploadsManager.gatingManager.shouldShowAdminDisabledBannerInHomeForUser(user)
    }

    override fun createBanner(
        bannerContext: BannerContext,
        userset: DbxUserset,
        proceedListener: View.OnClickListener,
        dismissListener: View.OnClickListener
    ): CameraUploadsAdminDisabeldControlBannerHeaderItem {

        return CameraUploadsAdminDisabeldControlBannerHeaderItem(
                BannerHeaderItem.HEADER_TYPE_ADMIN_DISABLED_CU_BANNER,
                proceedListener,
                dismissListener)
    }

    override fun logShowBanner(
        bannerContext: BannerContext,
        userset: DbxUserset
    ) {
        val user = checkNotNull(bannerContext.user) { "User is null!" }
        logBannerAction(user,
                CameraUploadsBannersEvents.CameraUploadsBannerType.ADMIN_DISABLE_BANNER,
                CameraUploadsBannersEvents.CameraUploadsBannerAction.SHOWN)
    }

    override fun handleBannerAction(
        bannerContext: BannerContext,
        userset: DbxUserset
    ) {
        val user = checkNotNull(bannerContext.user) { "User is null!" }
        val activity = bannerContext.context as BaseActivity
        activitiesHelper.showHelpArticle(activity, WebUrl.HELP_CAMERA_UPLOAD, user.logger)

        logBannerAction(user,
                CameraUploadsBannersEvents.CameraUploadsBannerType.ADMIN_DISABLE_BANNER,
                CameraUploadsBannersEvents.CameraUploadsBannerAction.TAPPED)
    }

    override fun handleDismissAction(
        bannerContext: BannerContext,
        userset: DbxUserset
    ) {
        val user = checkNotNull(bannerContext.user) { "User is null!" }
        user.userProperties.setShouldShowAdminDisabledBannerInHome(false)

        logBannerAction(user,
                CameraUploadsBannersEvents.CameraUploadsBannerType.ADMIN_DISABLE_BANNER,
                CameraUploadsBannersEvents.CameraUploadsBannerAction.DISMISSED)
    }

    private fun logBannerAction(
        user: DbxUser,
        cameraUploadsBannersType: CameraUploadsBannersEvents.CameraUploadsBannerType,
        cameraUploadsBannersAction: CameraUploadsBannersEvents.CameraUploadsBannerAction
    ) {
        CameraUploadsBannersEvents.CameraUploadsBannerStatus()
                .setCuBannerType(cameraUploadsBannersType)
                .setCuBannerAction(cameraUploadsBannersAction)
                .logTo(user.logger)
    }
}
