// dbx-min-coverage: 77% because some codes are unnecessary to test
/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */

package com.dropbox.android.banner

import android.view.View
import com.dropbox.android.activity.base.BaseActivity
import com.dropbox.android.browser.banners.BannerHeaderItem
import com.dropbox.android.browser.banners.CameraUploadsPromoBannerHeaderItem
import com.dropbox.android.camerauploads.CameraUploadsManager
import com.dropbox.android.user.DbxUser
import com.dropbox.android.user.DbxUserset
import com.dropbox.base.analytics.CameraUploadsBannersEvents
import com.dropbox.base.analytics.CameraUploadsFeatureEvents

// All Camera Uploads Promo Banner Types
private enum class PromoBannerType {
    ENABLE_PROMO_BANNER,
    UPSELL_PROMO_BANNER,
    NONE
}

open class CameraUploadsPromoBanner internal constructor() :
        BaseBanner<CameraUploadsPromoBannerHeaderItem>() {

    private var currentPromoBannerType: PromoBannerType = PromoBannerType.NONE

    private fun getPromoBannerType(
        cameraUploadsManager: CameraUploadsManager,
        dbxUser: DbxUser
    ): PromoBannerType = when {
            cameraUploadsManager.gatingManager.shouldShowEnablePromoBannerInHomeForUser(dbxUser)
            -> PromoBannerType.ENABLE_PROMO_BANNER
            cameraUploadsManager.gatingManager.shouldShowUpsellPromoBannerInHomeForUser(dbxUser)
            -> PromoBannerType.UPSELL_PROMO_BANNER
            else -> PromoBannerType.NONE
        }

    override fun bannerActionShouldDismissBanner(): Boolean {
        return false
    }

    override fun shouldShowBanner(
        bannerContext: BannerContext,
        userset: DbxUserset
    ): Boolean {
        val user = bannerContext.user ?: return false

        val cameraUploadsManager = bannerContext.cameraUploadsManager
        currentPromoBannerType = getPromoBannerType(cameraUploadsManager, user)

        return currentPromoBannerType != PromoBannerType.NONE
    }

    override fun createBanner(
        bannerContext: BannerContext,
        userset: DbxUserset,
        proceedListener: View.OnClickListener,
        dismissListener: View.OnClickListener
    ): CameraUploadsPromoBannerHeaderItem {

        return CameraUploadsPromoBannerHeaderItem(
                BannerHeaderItem.HEADER_TYPE_CAMERA_UPLOADS_BANNER,
                true,
                proceedListener,
                dismissListener)
    }

    override fun logShowBanner(
        bannerContext: BannerContext,
        userset: DbxUserset
    ) {
        val user = checkNotNull(bannerContext.user) { "User is null!" }
        logBannerAction(user,
                adlBannerType(currentPromoBannerType),
                CameraUploadsBannersEvents.CameraUploadsBannerAction.SHOWN)
    }

    override fun handleBannerAction(
        bannerContext: BannerContext,
        userset: DbxUserset
    ) {

        val user = checkNotNull(bannerContext.user) { "User is null!" }

        val context = bannerContext.context
        val activity = context as BaseActivity

        when (getPromoBannerType(bannerContext.cameraUploadsManager, user)) {
            PromoBannerType.ENABLE_PROMO_BANNER, PromoBannerType.UPSELL_PROMO_BANNER ->
                bannerContext.cameraUploadsManager.enableCameraUploadsOrShowGatingUI(
                        user,
                        activity,
                        CameraUploadsFeatureEvents.CameraUploadsEnableSource.HOME_BANNER,
                        null,
                        Runnable { this.hideBanner() })
            PromoBannerType.NONE -> {
                // Do nothing on PromoBannerType.NONE
            }
        }

        logBannerAction(user,
                adlBannerType(currentPromoBannerType),
                CameraUploadsBannersEvents.CameraUploadsBannerAction.TAPPED)
    }

    override fun handleDismissAction(
        bannerContext: BannerContext,
        userset: DbxUserset
    ) {
        val user = checkNotNull(bannerContext.user) { "User is null!" }
        user.userProperties.setCameraUploadPromoBannerDismissed(true)

        logBannerAction(user,
                adlBannerType(currentPromoBannerType),
                CameraUploadsBannersEvents.CameraUploadsBannerAction.DISMISSED)
    }

    private fun logBannerAction(
        user: DbxUser,
        cameraUploadsBannersType: CameraUploadsBannersEvents.CameraUploadsBannerType,
        cameraUploadsBannersAction: CameraUploadsBannersEvents.CameraUploadsBannerAction
    ) {
        CameraUploadsBannersEvents.CameraUploadsBannerStatus()
                .setCuBannerType(cameraUploadsBannersType)
                .setCuBannerAction(cameraUploadsBannersAction)
                .logTo(user.logger)
    }

    private fun adlBannerType(
        promoBanner: PromoBannerType
    ): CameraUploadsBannersEvents.CameraUploadsBannerType {
        return when (promoBanner) {
            PromoBannerType.ENABLE_PROMO_BANNER
            -> CameraUploadsBannersEvents.CameraUploadsBannerType.ENABLE_BANNER
            PromoBannerType.UPSELL_PROMO_BANNER
            -> CameraUploadsBannersEvents.CameraUploadsBannerType.UPSELL_BANNER
            PromoBannerType.NONE
            -> CameraUploadsBannersEvents.CameraUploadsBannerType.UNKNOWN
        }
    }
}
