/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */

package com.dropbox.android.camerauploads

import com.dropbox.android.user.DbxUser

/**
 * Class that handles all decision making around which users can enable CU and which UI elements
 * those users can see.
 */
interface CameraUploadsGatingManager {
    /**
     * @return the {@link DbxUser} that can interact with the CU feature, this is equivalent to the
     * legacy 'photoUser' concept. Currently, as long as the users have the capability equals to
     * CAN_BECOME_AVAILABLE or AVAILABLE will be able to interact with CU.
     * TODO(ageiduschek): Deprecate all external references to this. This should be an
     * implementation detail of this class.
     */
    val cuUser: DbxUser?

    /**
     * @return whether the passed userState can enable CU. Will also return true if CU is currently
     * enabled.
     */
    fun canEnableCuForUser(user: DbxUser?): Boolean

    /**
     * @return whether the passed userState could enable CU if they upgraded or linked a desktop client.
     */
    fun canEnableCuWithUpgradeForUser(user: DbxUser?): Boolean

    /**
     * @return whether we can show the CU status bar in the photos tab for the given user.
     * This will return true for users that are a) eligible for CU or eligible with upgrade and
     * b) won't be shown CU status UI in the Home tab.
     */
    fun shouldShowCuUIInPhotosTabForUser(user: DbxUser?): Boolean

    /**
     * @return whether the passed user would be able to enable CU if they upgraded / linked a
     * desktop, but are currently not eligible, so we should show upsell UI in Home.
     */
    fun shouldShowUpsellPromoBannerInHomeForUser(user: DbxUser?): Boolean

    /**
     * @return whether the passed user has CU disabled can currently enable CU, so we can show
     * UI prompting the user to enable CU in Home.
     */
    fun shouldShowEnablePromoBannerInHomeForUser(user: DbxUser?): Boolean

    /**
     * @return whether the passed user would be able to enable CU if they upgraded / linked a
     * desktop, but are currently not eligible, so we should show upsell UI in Home.
     */
    fun shouldShowCuStatusUIInHomeForUser(user: DbxUser?): Boolean

    /**
     * @return whether we should show the admin disabled banner in Home.
     */
    fun shouldShowAdminDisabledBannerInHomeForUser(user: DbxUser?): Boolean
}
