// dbx-min-coverage: 0% because it's a helper for UI logic
package com.dropbox.android.camerauploads

import android.content.Context
import android.support.v4.app.FragmentActivity
import com.dropbox.android.R
import com.dropbox.android.activity.DropboxBrowser
import com.dropbox.android.activity.base.BaseActivity
import com.dropbox.android.activity.payment.PaymentCCWebviewActivity
import com.dropbox.android.activity.payment.PaymentSelectorActivity
import com.dropbox.android.user.DbxUser
import com.dropbox.android.user.UserCapability
import com.dropbox.android.util.Activities
import com.dropbox.android.util.web.WebUrl
import com.dropbox.base.analytics.Analytics
import com.dropbox.base.analytics.CameraUploadsFeatureEvents
import com.dropbox.base.analytics.CameraUploadsStatusIndicatorEvents
import com.dropbox.base.device.DeviceFeatures
import com.dropbox.product.dbapp.camera_upload.cu_ui.DbxCameraUploadsFeatureStatus
import com.dropbox.core.ui.util.DbxAlertDialogBuilder

class CameraUploadsUserStatusSelectionHelper(
    private val context: Context,
    private val activity: FragmentActivity,
    private val user: DbxUser,
    private val cameraUploadsManager: CameraUploadsManager
) {
    private val deviceFeatures = DeviceFeatures(activity.applicationContext)

    fun onCUStatusBarClicked(status: DbxCameraUploadsFeatureStatus) {
        when (status) {
            DbxCameraUploadsFeatureStatus.VIDEO_UPLOADS_OFF ->
                if (!user.userProperties.cameraUploadVideosEnabled()) {
                    showTurnOnVideoUploadsDialog()
                }
            DbxCameraUploadsFeatureStatus.WAITING_FOR_WIFI_FOR_VIDEOS ->
                if (user.userProperties.cameraUploadLimitVideoUploadsOverCell()) {
                    showTurnOnVideoUploadsOverCell()
                }
            DbxCameraUploadsFeatureStatus.CANNOT_UPLOAD_UNPLUGGED ->
                if (user.userProperties.cameraUploadOnlyWhenCharging()) {
                    showTurnOffOnlyUploadWhileChargingDialog()
                }
            DbxCameraUploadsFeatureStatus.OVER_DROPBOX_QUOTA ->
                if (user.capabilitiesManager
                                .capabilityIsAvailable(UserCapability.IncreaseQuota::class.java)) {
                    PaymentSelectorActivity.showPaymentActivity(
                            activity,
                            PaymentCCWebviewActivity.UpgradeSource.OVER_QUOTA_CAMERA_UPLOAD)
                }
            DbxCameraUploadsFeatureStatus.WAITING_FOR_WIFI ->
                // TODO(ageiduschek): Once legacy CU goes away, we should never reach this case
                // with `cameraUploadUse3G() == true`
                if ((!user.userProperties.cameraUploadUse3G() && deviceFeatures.hasMobileData())) {
                    showTurnOnCellDialog()
                }
            DbxCameraUploadsFeatureStatus.DISABLED ->
                cameraUploadsManager.enableCameraUploadsOrShowGatingUI(
                        user,
                        activity as BaseActivity,
                        CameraUploadsFeatureEvents.CameraUploadsEnableSource.PROMPT,
                        null /*snackbarProvider*/,
                        null /*successCallback*/)
            DbxCameraUploadsFeatureStatus.SCAN_FAILED -> {
                Activities.showHelpArticle(
                        activity, WebUrl.HELP_CAMERA_UPLOAD_SCAN_FAILED, user.logger)
            }
            DbxCameraUploadsFeatureStatus.UPLOADING,
            DbxCameraUploadsFeatureStatus.ALL_DONE -> {
                val path = cameraUploadsManager.cuFolderPathAbsoluteForUser(user)
                if (path != null) {
                    activity.startActivity(DropboxBrowser.goToFolderIntent(path, user.id))
                }
            }
            DbxCameraUploadsFeatureStatus.NOT_STARTED,
            DbxCameraUploadsFeatureStatus.NO_CONNECTION,
            DbxCameraUploadsFeatureStatus.WAITING_FOR_FASTER_NETWORK,
                // TODO(weilhuiz): Potentially also allow "Tap to change settings" when battery is
                // low, currently this behavior is not present in either legacy or V3 CU.
            DbxCameraUploadsFeatureStatus.LOW_BATTERY_CHARGING,
            DbxCameraUploadsFeatureStatus.LOW_BATTERY_NOT_CHARGING,
            DbxCameraUploadsFeatureStatus.WAITING_TO_UPLOAD,
            DbxCameraUploadsFeatureStatus.SCANNING_LIBRARY,
            DbxCameraUploadsFeatureStatus.PAUSED_FOR_PENDING_MANUAL_UPLOADS,
            DbxCameraUploadsFeatureStatus.REMOTE_PAUSED,
            DbxCameraUploadsFeatureStatus.OVER_BATTERY_QUOTA,
            DbxCameraUploadsFeatureStatus.ROAMING_PAUSED,
            DbxCameraUploadsFeatureStatus.PERMISSION_DENIED,
            DbxCameraUploadsFeatureStatus.NEED_SETTINGS_UPDATE,
            DbxCameraUploadsFeatureStatus.ICLOUD_SYNC_PAUSED -> {
            }
            else -> throw RuntimeException("Unexpected camera upload tracker state")
        }
        CameraUploadsStatusIndicatorEvents.Tapped()
                .setCuStatus(uploadStatusToIndicatorEventStatus(status))
                .logTo(user.logger)
    }

    fun onCUStatusBarDismissed(status: DbxCameraUploadsFeatureStatus) {
        user.userProperties.setHasDismissedCameraUploadStatusAndNotify(true)
        CameraUploadsStatusIndicatorEvents.Dismissed()
                .setCuStatus(uploadStatusToIndicatorEventStatus(status))
                .logTo(user.logger)
    }

    private fun uploadStatusToIndicatorEventStatus(status: DbxCameraUploadsFeatureStatus):
            CameraUploadsStatusIndicatorEvents.CameraUploadsStatus? {
        return when (status) {
            DbxCameraUploadsFeatureStatus.VIDEO_UPLOADS_OFF ->
                    CameraUploadsStatusIndicatorEvents.CameraUploadsStatus.VIDEO_UPLOADS_OFF
            DbxCameraUploadsFeatureStatus.DISABLED ->
                CameraUploadsStatusIndicatorEvents.CameraUploadsStatus.DISABLED
            DbxCameraUploadsFeatureStatus.NOT_STARTED ->
                CameraUploadsStatusIndicatorEvents.CameraUploadsStatus.NOT_STARTED
            DbxCameraUploadsFeatureStatus.SCANNING_LIBRARY ->
                CameraUploadsStatusIndicatorEvents.CameraUploadsStatus.SCANNING_LIBRARY
            DbxCameraUploadsFeatureStatus.UPLOADING ->
                CameraUploadsStatusIndicatorEvents.CameraUploadsStatus.UPLOADING
            DbxCameraUploadsFeatureStatus.ALL_DONE ->
                CameraUploadsStatusIndicatorEvents.CameraUploadsStatus.ALL_DONE
            DbxCameraUploadsFeatureStatus.PAUSED_FOR_PENDING_MANUAL_UPLOADS ->
                CameraUploadsStatusIndicatorEvents
                        .CameraUploadsStatus.PAUSED_FOR_PENDING_MANUAL_UPLOADS
            DbxCameraUploadsFeatureStatus.REMOTE_PAUSED ->
                CameraUploadsStatusIndicatorEvents.CameraUploadsStatus.REMOTE_PAUSED
            DbxCameraUploadsFeatureStatus.OVER_DROPBOX_QUOTA ->
                CameraUploadsStatusIndicatorEvents.CameraUploadsStatus.OVER_DROPBOX_QUOTA
            DbxCameraUploadsFeatureStatus.OVER_BATTERY_QUOTA ->
                CameraUploadsStatusIndicatorEvents.CameraUploadsStatus.OVER_BATTERY_QUOTA
            DbxCameraUploadsFeatureStatus.LOW_BATTERY_CHARGING ->
                CameraUploadsStatusIndicatorEvents.CameraUploadsStatus.LOW_BATTERY_CHARGING
            DbxCameraUploadsFeatureStatus.LOW_BATTERY_NOT_CHARGING ->
                CameraUploadsStatusIndicatorEvents.CameraUploadsStatus.LOW_BATTERY_NOT_CHARGING
            DbxCameraUploadsFeatureStatus.CANNOT_UPLOAD_UNPLUGGED ->
                CameraUploadsStatusIndicatorEvents.CameraUploadsStatus.CANNOT_UPLOAD_UNPLUGGED
            DbxCameraUploadsFeatureStatus.NO_CONNECTION ->
                CameraUploadsStatusIndicatorEvents.CameraUploadsStatus.NO_CONNECTION
            DbxCameraUploadsFeatureStatus.WAITING_FOR_WIFI ->
                CameraUploadsStatusIndicatorEvents.CameraUploadsStatus.WAITING_FOR_WIFI
            DbxCameraUploadsFeatureStatus.WAITING_FOR_WIFI_FOR_VIDEOS ->
                CameraUploadsStatusIndicatorEvents.CameraUploadsStatus.WAITING_FOR_WIFI_FOR_VIDEOS
            DbxCameraUploadsFeatureStatus.WAITING_FOR_WIFI_TO_SCAN_CLOUD_LIBRARY ->
                CameraUploadsStatusIndicatorEvents
                        .CameraUploadsStatus.WAITING_FOR_WIFI_TO_SCAN_CLOUD_LIBRARY
            DbxCameraUploadsFeatureStatus.WAITING_FOR_FASTER_NETWORK ->
                CameraUploadsStatusIndicatorEvents.CameraUploadsStatus.WAITING_FOR_FASTER_NETWORK
            DbxCameraUploadsFeatureStatus.WAITING_TO_UPLOAD ->
                CameraUploadsStatusIndicatorEvents.CameraUploadsStatus.WAITING_TO_UPLOAD
            DbxCameraUploadsFeatureStatus.ROAMING_PAUSED ->
                CameraUploadsStatusIndicatorEvents.CameraUploadsStatus.ROAMING_PAUSED
            DbxCameraUploadsFeatureStatus.SCAN_FAILED ->
                CameraUploadsStatusIndicatorEvents.CameraUploadsStatus.SCAN_FAILED
            DbxCameraUploadsFeatureStatus.PERMISSION_DENIED ->
                CameraUploadsStatusIndicatorEvents.CameraUploadsStatus.PERMISSION_DENIED
            DbxCameraUploadsFeatureStatus.NEED_SETTINGS_UPDATE ->
                CameraUploadsStatusIndicatorEvents.CameraUploadsStatus.NEED_SETTINGS_UPDATE
            DbxCameraUploadsFeatureStatus.ICLOUD_SYNC_PAUSED ->
                // this event would never happen
                CameraUploadsStatusIndicatorEvents.CameraUploadsStatus.ICLOUD_SYNC_PAUSED
            else -> throw RuntimeException("Unexpected camera upload tracker state")
        }
    }

    private fun showTurnOnVideoUploadsDialog() =
            DbxAlertDialogBuilder(context)
                    .setCancelable(true)
                    .setTitle(R.string.turn_on_video_uploads_dialog_title)
                    .setMessage(R.string.turn_on_video_uploads_dialog_message)
                    .setPositiveButton(
                            R.string.turn_on_video_uploads_dialog_positive_button
                    ) { _, _ ->
                        cameraUploadsManager.setVideosEnabled(user, true /*videoEnabled*/)
                        Analytics.PREF_CHANGED()
                                .set("cameraupload.enablevideouploads", true)
                                .logTo(user.logger)
                    }
                    .setNegativeButton(
                            R.string.turn_on_video_uploads_dialog_negative_button,
                            null /*listener*/)
                    .show()

    private fun showTurnOnVideoUploadsOverCell() =
            DbxAlertDialogBuilder(context)
                    .setCancelable(true)
                    .setTitle(R.string.turn_on_video_uploads_over_cellular_dialog_title)
                    .setMessage(R.string.turn_on_video_uploads_over_cellular_dialog_message)
                    .setPositiveButton(
                            R.string
                                    .turn_on_video_uploads_over_cellular_dialog_positive_button
                    ) { _, _ ->
                        cameraUploadsManager.setLimitVideoUploadsOverCell(
                                user, false)
                        Analytics.PREF_CHANGED()
                                .set(
                                        "cameraupload.limitdataplanfilesize",
                                        false)
                                .logTo(user.logger)
                    }
                    .setNegativeButton(
                            R.string
                                    .turn_on_video_uploads_over_cellular_dialog_negative_button,
                            null)
                    .show()

    private fun showTurnOffOnlyUploadWhileChargingDialog() =
            DbxAlertDialogBuilder(context)
                    .setCancelable(true)
                    .setTitle(R.string.turn_off_only_upload_while_charging_dialog_title)
                    .setMessage(R.string.turn_off_only_upload_while_charging_dialog_message)
                    .setPositiveButton(
                            R.string
                                    .turn_off_only_upload_while_charging_dialog_positive_button
                    ) { _, _ ->
                        cameraUploadsManager.setEnabledOnlyWhenCharging(
                                user, false)
                        Analytics.PREF_CHANGED()
                                .set("cameraupload.onlywhencharging", false)
                                .logTo(user.logger)
                    }
                    .setNegativeButton(
                            R.string
                                    .turn_off_only_upload_while_charging_dialog_negative_button,
                            null /*listener*/)
                    .show()

    // Please note that even if camera upload over 3G is enabled the upload queue can still
    // be waiting for wifi if there is a 3g limit and there is a large file in the front
    // of the queue.
    private fun showTurnOnCellDialog() =
            DbxAlertDialogBuilder(context)
                    .setCancelable(true)
                    .setTitle(R.string.turn_on_cellular_dialog_title)
                    .setMessage(R.string.turn_on_cellular_dialog_message)
                    .setPositiveButton(
                            R.string.turn_on_cellular_dialog_positive_button
                    ) { _, _ ->
                        // Set the 3g limit to true as stated in the dialog message.
                        cameraUploadsManager.setLimitVideoUploadsOverCell(
                                user, true)
                        cameraUploadsManager.setUse3G(user, true)
                        Analytics.PREF_CHANGED()
                                .set("cameraupload.usedataplan", true)
                                .logTo(user.logger)
                    }
                    .setNegativeButton(
                            R.string.turn_on_cellular_dialog_negative_button, null)
                    .show()
}
