// dbx-min-coverage: 29% because we need to be able to inject DbxCloudMessagingManager
// and AnalyticsLogger in order to write tests
package com.dropbox.android.cloudmessaging

import com.dropbox.android.DropboxApplication
import com.dropbox.base.analytics.AnalyticsLogger
import com.dropbox.base.build_configuration.DevBuild
import com.dropbox.base.oxygen.DbxLog
import com.google.firebase.messaging.FirebaseMessagingService
import com.google.firebase.messaging.RemoteMessage
import org.json.simple.JSONArray
import org.json.simple.parser.JSONParser
import org.json.simple.parser.ParseException

private val TAG = DbxCloudMessagingService::class.java.name
private const val ATTR_REFRESH = "refresh"
private const val ATTR_TARGET_USER_ID = "target_user_id"

/**
 * This service is responsible for actually handling the FCM messages.
 */
class DbxCloudMessagingService : FirebaseMessagingService() {

    private lateinit var cloudMessagingManager: DbxCloudMessagingManager
    private lateinit var analyticsLogger: AnalyticsLogger

    override fun onCreate() {
        cloudMessagingManager = DropboxApplication.getCloudMessagingManager(this)
        analyticsLogger = DropboxApplication.getDefaultEventLogger(this)

        super.onCreate()
    }

    override fun onNewToken(token: String?) {
        DbxLog.d(TAG, "Received new token: ${token ?: "null"}")
        cloudMessagingManager.sendRegistrationToServer(token)
    }

    override fun onMessageReceived(remoteMessage: RemoteMessage?) {
        if (remoteMessage == null) {
            DbxLog.d(TAG, "Received null message")
            return
        }

        if (DevBuild.IS_DEV_BUILD) {
            DbxLog.d(TAG, "Received FCM data: ${remoteMessage.data}")
        }

        val data = remoteMessage.data
        val refreshData = data[ATTR_REFRESH]

        if (refreshData != null) {
            val p = JSONParser()
            try {
                val refreshDataJson = p.parse(refreshData) as JSONArray
                if (refreshDataJson.contains("user")) {
                    val targetUserId = data[ATTR_TARGET_USER_ID]
                    if (targetUserId != null) {
                        cloudMessagingManager.triggerNotificationsFetch(targetUserId)
                    } else {
                        DbxLog.d(TAG, "Ignore FCM notification for NULL user id")
                    }
                }
            } catch (e: ParseException) {
                DbxLog.d(TAG, e.localizedMessage, e)
            } catch (e: ClassCastException) {
                DbxLog.d(TAG, e.localizedMessage, e)
            }
        }
    }
}
