/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */

package com.dropbox.android.content.camerauploads

import android.support.annotation.CallSuper
import com.dropbox.android.content.ViewModel
import com.dropbox.android.content.ViewModelType
import com.dropbox.product.dbapp.camera_upload.cu_engine.CameraUploadUtils.LocalIdBase
import com.dropbox.product.dbapp.camera_upload.cu_ui.DbxCameraUploadsFeatureStatus
import com.google.common.base.Objects

/**
 * View model containing the consolidated information from the CU business logic.
 */
open class CameraUploadsViewModel protected constructor(
    builder: CameraUploadsViewModel.AbstractBuilder<*, *>,
    private val cuUserStatus: CameraUploadsUserStatus = builder.cuUserStatus
) : ViewModel(builder) {

    val cuStatus: DbxCameraUploadsFeatureStatus = cuUserStatus.cuSnapshot.status
    val numUserPendingUploads: Int = cuUserStatus.cuSnapshot.numUserPendingUploads
    val numQueuedVideoUploads: Int = cuUserStatus.cuSnapshot.numQueuedVideoUploads
    val currentUploadId: String? = cuUserStatus.cuSnapshot.currentUploadInfo?.localId
    val currentUploadFileSize: Long? = cuUserStatus.cuSnapshot.currentUploadInfo?.uploadFileSize
    val isCurrentUploadVideo: Boolean = cuUserStatus.cuSnapshot.currentUploadInfo?.isVideo ?: false
    val uploadProgress: Double? = cuUserStatus.cuSnapshot.currentUploadInfo?.uploadProgress?.let {
        it * 100
    }

    val userCanUpgrade: Boolean = cuUserStatus.userCanUpgrade
    val userCanEnableCameraUploads: Boolean = cuUserStatus.userCanEnableCameraUploads
    val shouldShowConfirmTranscodeSettingBannerInHome: Boolean =
            cuUserStatus.shouldShowConfirmTranscodeSettingBannerInHome
    val shouldShowStatusBarInHome: Boolean = cuUserStatus.shouldShowStatusBarInHome
    val shouldShowEnableBannerInHome: Boolean = cuUserStatus.shouldShowEnableBannerInHome
    val shouldShowUpsellBannerInHome: Boolean = cuUserStatus.shouldShowUpsellBannerInHome
    val cuControlIcon: CameraUploadsControlIcon = cuUserStatus.cuControlIcon
    val cuStatusTitle: String get() {
        val remainingText = if (numUserPendingUploads > 0) " ($numUserPendingUploads left)" else ""
        return cuUserStatus.cuStatusTitle + remainingText
    }

    val cuStatusSubtitle: String = cuUserStatus.cuStatusSubtitle
    val cuStatusSubtitleColor: Int = cuUserStatus.cuStatusSubtitleColor
    val cuStatusIcon: Int = cuUserStatus.cuStatusIcon

    val currentUploadURI: String? get() = currentUploadId?.let {
        with(LocalIdBase.fromString(it)) { "$uri/$id" }
    }

    @CallSuper
    override fun equals(other: Any?): Boolean {
        if (this === other) {
            return true
        }

        if (!super.equals(other) || other !is CameraUploadsViewModel) {
            return false
        }

        return cuUserStatus == other.cuUserStatus
    }

    @CallSuper
    override fun hashCode(): Int {
        return Objects.hashCode(super.hashCode(), cuUserStatus.hashCode())
    }

    /**
     * Builder for [CameraUploadsViewModel].
     */
    class Builder : AbstractBuilder<CameraUploadsViewModel, Builder>() {
        override fun buildForBaseClass(): CameraUploadsViewModel {
            return CameraUploadsViewModel(this)
        }
    }

    /**
     * Base class for builders of [T].
     */
    abstract class AbstractBuilder<T : CameraUploadsViewModel,
            B : CameraUploadsViewModel.AbstractBuilder<T, B>>
    protected constructor() : ViewModel.AbstractBuilder<T, B>() {
        internal lateinit var cuUserStatus: CameraUploadsUserStatus

        init {
            mViewModelType = ViewModelType.CAMERA_UPLOADS_VIEW_MODEL
        }

        @CallSuper
        override fun injectFrom(instance: T?): B {
            return super.injectFrom(instance)
        }
    }
}
