// dbx-min-coverage: 0% because grandfathered
package com.dropbox.android.dev

import android.content.ComponentName
import android.content.Intent
import android.graphics.Color
import android.graphics.Typeface
import android.net.Uri
import android.os.Bundle
import android.view.Gravity
import android.view.View
import android.view.ViewGroup
import android.widget.AdapterView
import android.widget.AdapterView.OnItemSelectedListener
import android.widget.ArrayAdapter
import android.widget.Button
import android.widget.CheckBox
import android.widget.FrameLayout
import android.widget.LinearLayout
import android.widget.LinearLayout.LayoutParams
import android.widget.ScrollView
import android.widget.Spinner
import android.widget.TextView
import com.dropbox.android.DropboxApplication
import com.dropbox.android.activity.CameraUploadSettingsActivity
import com.dropbox.android.activity.DevCUTransitionActivity
import com.dropbox.android.activity.DevTiburonActivity
import com.dropbox.android.activity.StormcrowOverrideFragment
import com.dropbox.android.activity.UserCapabilityOverrideFragment
import com.dropbox.android.activity.base.BaseActivity
import com.dropbox.android.activity.base.BaseActivityHelper
import com.dropbox.android.activity.payment.PaymentCCWebviewActivity.UpgradeSource
import com.dropbox.android.activity.payment.PaymentSelectorActivity
import com.dropbox.android.migrate.CompanyDropboxMigrationActivity
import com.dropbox.android.openwith.DeviceOpenWithManager
import com.dropbox.android.preference.ManageSubscriptionActivity
import com.dropbox.android.previewable.DevicePreviewableManagerImpl
import com.dropbox.android.provider.PhotosProvider.PhotosChangeNotifier
import com.dropbox.android.service.AccountInfoManager
import com.dropbox.android.settings.GlobalProperties
import com.dropbox.android.settings.GlobalProperties.WopiSetting
import com.dropbox.android.settings.SharedProperties
import com.dropbox.android.user.DbxUser
import com.dropbox.android.user.DbxUser.UserRole
import com.dropbox.android.user.DbxUserManager
import com.dropbox.android.user.DbxUserset
import com.dropbox.android.util.AppInfoUtils
import com.dropbox.android.util.InternalBuild
import com.dropbox.android.util.ToastFactory
import com.dropbox.base.build_configuration.DevBuild
import com.dropbox.base.error.DbxException
import com.dropbox.base.json.JsonKeyValueOverride
import com.dropbox.base.oxygen.DbxLog
import com.dropbox.base.util.SystemInfo
import com.dropbox.core.legacy_api.exception.DropboxException
import com.dropbox.core.stormcrow.NoauthStormcrow
import com.dropbox.core.ui.util.DbxAlertDialogBuilder
import com.dropbox.hairball.database.DBTablePhotos
import com.dropbox.internalclient.UserApi
import com.dropbox.product.dbapp.syncapi_dbapp_code_gen.DbappDebugHelper
import com.squareup.leakcanary.RefWatcher
import kotlin.concurrent.thread

/** Debug screen  */
class DevControllerActivity : BaseActivity() {

    private lateinit var mUserManager: DbxUserManager
    private lateinit var mDeviceStormcrow: NoauthStormcrow
    private lateinit var mRefWatcher: RefWatcher
    private lateinit var mGlobalProperties: GlobalProperties
    private lateinit var mParent: ViewGroup

    companion object {
        private val TAG = DevControllerActivity::class.java.name

        /** Used to intentionally leak an object for debug purposes  */
        private var sLeakHolder: Any? = null
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        if (!InternalBuild.IS_INTERNAL_BUILD) {
            finish()
            return
        } else {
            // This else statement is not necessary for correctness, but it will cause the following
            // code not to be compiled in release builds.

            mParent = createScrollingContainer()

            if (DropboxApplication.isAppBeingMonkeyTested(this)) {
                throw RuntimeException("Monkey isn't allowed in DevControllerActivity!")
            }
            mDeviceStormcrow = DropboxApplication.getDeviceStormcrow(this)
            mUserManager = DropboxApplication.getUserManager(this)
            mRefWatcher = DropboxApplication.getRefWatcher(this)
            mGlobalProperties = DropboxApplication.getGlobalProperties(this)

            val users = mUserManager.users
            val user = users?.getPreferredUser(UserRole.PERSONAL)
            val personalUser = users?.getUserByRole(UserRole.PERSONAL)
            val businessUser = users?.getUserByRole(UserRole.BUSINESS)

            mParent.addView(createDivider())
            stormcrowSection(users)
            mParent.addView(createDivider())

            button("Refresh PreviewableManager") {
                thread(start = true) {
                    DbxLog.d(TAG, "Refreshing PreviewableManager")
                    val previewableManager = DropboxApplication
                            .getDevicePreviewableManager(this) as DevicePreviewableManagerImpl
                    previewableManager.forceUpdateInfo()
                }
            }

            mParent.addView(createDivider())

            button("Crash") {
                throw RuntimeException("Forced crash!")
            }

            button("C++ SIGABRT") {
                DbappDebugHelper.forceNativeCrash()
            }

            // LeakCanary is only included in debug builds, not internal builds
            if (DevBuild.IS_DEV_BUILD) {
                button("Leak!") {
                    val leakThing = Any()
                    mRefWatcher.watch(leakThing, "Dummy intentional leak")
                    sLeakHolder = leakThing
                    ToastFactory.showText(this, "wait for it...")
                }
            }

            mParent.addView(createDivider())

            if (user != null) {
                sharedPropertiesSection(user.identityState.sharedProperties)

                mParent.addView(createDivider())
                miscellaneousSection(user)

                button("Mark all offline items as dirty") {
                    thread(start = true) {
                        users.allUsers().forEach { user ->
                            user.metadataManager.markAllOfflineFilesAsDirty()
                        }
                    }
                }
                mParent.addView(createDivider())
            }

            cameraUploadsSections(users)

            if (users != null) {
                mParent.addView(createDivider())
                notificationSection(users)

                mParent.addView(createDivider())
                openWithSection(users)
            }

            if (user != null) {
                mParent.addView(createDivider())
                button("Clear all photos (unsafe)") {
                    val db = user.databaseHelper.writableDatabase
                    db.delete(DBTablePhotos.NAME, null, null)
                    val notifier = PhotosChangeNotifier(user.id, this.contentResolver)
                    notifier.notifyChange()
                    user.userProperties.removeAllPhotosCursor()
                }
            }

            mParent.addView(createDivider())
            text("Flows", true)

            if (user != null) {
                button("Launch QRAuth Activity") {
                    val riIntent = DropboxApplication.getGrowthExperiments(this)
                            .clientLinkFlowExperiment
                            .getAuthFlowLaunchIntent(this, user.id, "DevActivity")
                    startActivity(riIntent)
                }
            }

            if (users != null) {
                mParent.addView(createDivider())
                tutorialsSection(users)
            }

            if (user != null) {
                mParent.addView(createDivider())
                text("Prompt:", true)
                promptSection(user)
            }

            if (users != null) {
                mParent.addView(createDivider())
                settingsSection(users)
            }

            mParent.addView(createDivider())
            mslSection(user)

            if (personalUser != null) {
                mParent.addView(createDivider())
                upgradeSection(personalUser)
            }

            mParent.addView(createDivider())
            appLinkSection()

            mParent.addView(createDivider())
            cdmSection(businessUser)

            mParent.addView(createDivider())
            deviceInfoSection()
        }
    }

    private fun cameraUploadsSections(users: DbxUserset?) {
        users?.getUserByRole(UserRole.PERSONAL)?.let {
            text("Camera uploads (Personal)", true)
            cameraUploadsSection(it)
        }

        users?.getUserByRole(UserRole.BUSINESS)?.let {
            text("Camera uploads (Business)", true)
            cameraUploadsSection(it)
        }
    }

    private fun cameraUploadsSection(user: DbxUser) {
        val cameraUploadManager = DropboxApplication.getCameraUploadsManager(this)
        button("CU Transition") {
            startActivity(
                    DevCUTransitionActivity.getLaunchIntent(this, user.id))
        }

        button("CU settings") {
            startActivity(CameraUploadSettingsActivity.getLaunchIntent(
                    this, user.id,
                    CameraUploadSettingsActivity.CameraUploadSettingsActivityLaunchSource
                            .DEBUG_CONTROLLER))
        }

        if (user.userProperties.cameraUploadEnabled()) {
            val dialogBuilder = DbxAlertDialogBuilder(this)
                    .setTitle("Failed to run CU consistency checker")
                    .setPositiveButton("Ok") { _, _ -> }

            button("Reset the current Consistency Checker states") {
                try {
                    cameraUploadManager.debugResetCUConsistencyCheckerStates(user)
                } catch (e: DbxException) {
                    dialogBuilder.setMessage(e.message).show()
                }
            }

            button("Reset and start a new consistency check") {
                try {
                    cameraUploadManager.debugResetAndStartCUConsistencyChecker(user)
                } catch (e: DbxException) {
                    dialogBuilder.setMessage(e.message).show()
                }
            }
        }
    }

    private fun stormcrowSection(users: DbxUserset?) {
        text("Stormcrow", true)

        button("Stormcrow overrides") {
            val frag = StormcrowOverrideFragment()
            frag.show(this, supportFragmentManager)
        }

        button("Update Stormcrow") {
            // `getStormcrowFromServer` is a method, not a property
            @Suppress("UsePropertyAccessSyntax")
            try {
                mDeviceStormcrow.asStormcrowBase().getStormcrowFromServer()
                users?.allUsers()?.forEach { user ->
                    user.stormcrow.asStormcrowBase().getStormcrowFromServer()
                }
            } catch (e: DbxException) {
                throw RuntimeException(e)
            }
        }
    }

    private fun sharedPropertiesSection(sharedProps: SharedProperties) {
        text("Home", true)
        button("Show starred when empty") {
            sharedProps.homeHideStarredEmpty = false
        }

        button("Redo Recents Onboarding") {
            sharedProps.seenRecentsOnboarding = false
        }
    }

    private fun miscellaneousSection(user: DbxUser) {
        button("User capability overrides") {
            val frag = UserCapabilityOverrideFragment()
            frag.show(this, supportFragmentManager)
        }

        button("Invalid API tokens") {
            thread(start = true) {
                try {
                    DbxLog.d(TAG, "Invalidating tokens..")
                    user.api.invalidateTokens()
                    DbxLog.d(TAG, "Tokens invalidated.")
                    runOnUiThread {
                        ToastFactory.showText(this, "DEV: Tokens invalidated.")
                    }
                } catch (e: DropboxException) {
                    DbxLog.d(TAG, "Failed to invalidate tokens", e)
                    runOnUiThread {
                        ToastFactory.showText(this, "DEV: Failed to invalidate tokens: " + e)
                    }
                }
            }
        }

        button("Open legacy link") {
            startLegacyShortcutIntent()
        }

        button("Tiburon") {
            startActivity(DevTiburonActivity.getLaunchIntent(this, user.id))
        }
    }

    private fun notificationSection(users: DbxUserset) {
        users.getUserByRole(UserRole.PERSONAL)?.let { personalUser ->
            button("Add Notification for Personal") {
                personalUser.testNotificationProvider.addNotification()
            }
        }

        users.getUserByRole(UserRole.BUSINESS)?.let { businessUser ->
            button("Add Notification for Business") {
                businessUser.testNotificationProvider.addNotification()
            }
        }
    }

    private fun openWithSection(users: DbxUserset) {
        text("Open With Settings:", true)

        val openwithSpinner = createSpinner(
                listOf(
                        "Auto-detect Google Play",
                        "Force report Google Play",
                        "Force report NO Google Play"),
                0)
        mParent.addView(openwithSpinner)

        button("Refresh OpenWith") {
            thread(start = true) {
                DbxLog.d(TAG, "Refreshing OpenWith")

                val marketName: String? =
                        when (openwithSpinner.selectedItemId.toInt()) {
                            0 -> {
                                // auto-detect market
                                users.allUsers().forEach { user ->
                                    user.openWithManager
                                            .forceUpdateInfo()
                                }
                                return@thread
                            }
                            1 -> DeviceOpenWithManager.DISTRO_VALUE_GOOGLE_PLAY
                            2 -> null
                            else -> throw IllegalStateException(
                                    "unexpected openwithSpinner option"
                            )
                        }
                users.allUsers().forEach { user ->
                    user.openWithManager
                            .forceUpdateInfo(marketName)
                }
            }
        }

        button("Reset tooltip display count") {
            users.identityState.deviceOpenWithManager.resetAllPromotionDisplayCounts()
        }

        val wopiSpinner = createSpinner(
                listOf(
                        "Use WOPI when appropriate",
                        "Always use WOPI",
                        "Never use WOPI"),
                0)
        when (mGlobalProperties.wopiSetting) {
            WopiSetting.USE_WOPI_WHEN_AVAILABLE -> wopiSpinner.setSelection(0)
            WopiSetting.ALWAYS_USE_WOPI -> wopiSpinner.setSelection(1)
            WopiSetting.ALWAYS_USE_OPEN_WITH -> wopiSpinner.setSelection(2)
        }
        wopiSpinner.onItemSelectedListener = object : OnItemSelectedListener {
            override fun onItemSelected(
                adapterView: AdapterView<*>,
                view: View?,
                i: Int,
                l: Long
            ) {
                when (i) {
                    0 -> mGlobalProperties.wopiSetting = WopiSetting.USE_WOPI_WHEN_AVAILABLE
                    1 -> mGlobalProperties.wopiSetting = WopiSetting.ALWAYS_USE_WOPI
                    2 -> mGlobalProperties.wopiSetting = WopiSetting.ALWAYS_USE_OPEN_WITH
                    else -> throw IllegalStateException("Unexpected wopiSpinner option")
                }
            }

            override fun onNothingSelected(adapterView: AdapterView<*>) {}
        }
        mParent.addView(wopiSpinner)
    }

    private fun tutorialsSection(users: DbxUserset) {
        text("Tutorials", true)

        button("Reset Offline Folders Tutorial") {
            users.identityState.sharedProperties.seenOfflineFoldersTutorial = false
        }

        button("Reset Destination Picker Coachmark") {
            users.identityState.sharedProperties.seenDestinationPickerCoachmark = false
        }

        button("Reset Gallery Sharing Coachmark") {
            with(users.identityState.sharedProperties) {
                sharingTutorialLastShown = 0
                sharingTutorialDismissCount = 0
                isSharingTutorialComplete = false
            }
        }

        if (users.pairing != null) {
            button("Reset GAS Coachmark") {
                users.identityState
                        .sharedProperties.seenGasOnboarding = false
            }
        }
    }

    private fun promptSection(user: DbxUser) {
        mParent.addView(
                createCheckbox(
                        "Enable Fake Prompt Top Notification",
                        { user.promptCampaignManager.toggleFakePromptTopNotification() },
                        user.promptCampaignManager.fakePromptTopNotificationEnabled()))

        text("(fake campaign is active for 24 hours or until unchecked\"" +
                " + \" or until the app is restarted)")
    }

    private fun settingsSection(users: DbxUserset) {
        text(text = "Settings", bold = true)
        button("Clear all upsell flags") {
            users.allUsers().forEach {
                it.userProperties.isSettingsUpsellV1Dismissed = false
                it.userProperties.isSettingsUpsellV2Dismissed = false
            }
        }
    }

    private fun mslSection(user: DbxUser?) {
        text("Team Locked State", true)

        button("Set team to locked state") {
            UserApi.addAccountAndPairingInfoOverride(
                    JsonKeyValueOverride("team.is_locked_team", true))
        }

        button("Set quota to zero") {
            UserApi.addAccountAndPairingInfoOverride(
                    JsonKeyValueOverride("quota_info.quota", 0))
        }

        button("Reset overrides") {
            UserApi.clearAccountAndPairingInfoOverrides()
        }

        if (user != null) {
            button("Reload account info") {
                thread(start = true) {
                    try {
                        user.accountInfoManager.fetchAccountInfo(
                                AccountInfoManager.Staleness.NONE)
                    } catch (ex: DropboxException) {
                    }
                }
            }
        }
    }

    private fun upgradeSection(personalUser: DbxUser) {
        text("Upgrade Flow", true)
        button("Show payment success") {
            startActivity(DevPaymentSuccessActivity.getLaunchIntent(this, personalUser.id))
        }
        button("Show plus trial") {
            startActivity(PaymentSelectorActivity.getDevPlusTrialLaunchIntent(
                this, UpgradeSource.DEV_MENU))
        }
        button("Show plus trial (for new user)") {
            startActivity(PaymentSelectorActivity.getDevPlusTrialLaunchIntent(
                this, UpgradeSource.NEW_SIGN_UP))
        }
        button("Show plus trial (for existing user sign in)") {
            startActivity(PaymentSelectorActivity.getDevPlusTrialLaunchIntent(
                this, UpgradeSource.EXISTING_USER_SIGN_IN))
        }
        button("Clear list of users who saw sign in trial screen") {
            mGlobalProperties.clearUsersWhoSawSignInTrialOnThisDevice()
        }
        button("Show manage subscriptions") {
            startActivity(ManageSubscriptionActivity.getLaunchIntent(this))
        }
    }

    private fun appLinkSection() {
        button("Open 'Home'") {
            val url =
                "https://www.dropbox.com/dl/home/mobile?mobile_app_fallback"
            startActivity(Intent(Intent.ACTION_VIEW, Uri.parse(url)))
        }
        button("Open 'Connect a computer'") {
            val url =
                "https://www.dropbox.com/dl/connect-a-computer/install?mobile_app_fallback"
            startActivity(Intent(Intent.ACTION_VIEW, Uri.parse(url)))
        }
        button("Open 'Offline'") {
            val url =
                "https://www.dropbox.com/dl/offline/mobile?mobile_app_fallback"
            startActivity(Intent(Intent.ACTION_VIEW, Uri.parse(url)))
        }
        button("Open 'Camera Uploads'") {
            val url =
                "https://www.dropbox.com/dl/camera-uploads/mobile?mobile_app_fallback"
            startActivity(Intent(Intent.ACTION_VIEW, Uri.parse(url)))
        }
        button("Open 'Doc Scanner'") {
            val url =
                "https://www.dropbox.com/dl/doc-scanner/mobile?mobile_app_fallback"
            startActivity(Intent(Intent.ACTION_VIEW, Uri.parse(url)))
        }
        button("Open 'Files'") {
            val url =
                "https://www.dropbox.com/dl/files/mobile?mobile_app_fallback"
            startActivity(Intent(Intent.ACTION_VIEW, Uri.parse(url)))
        }
        button("Open 'Upgrade'") {
            val url =
                "https://www.dropbox.com/dl/upgrade/plans"
            startActivity(Intent(Intent.ACTION_VIEW, Uri.parse(url)))
        }
        button("Open bogus link") {
            val url =
                "https://www.dropbox.com/dl/bogus/link"
            startActivity(Intent(Intent.ACTION_VIEW, Uri.parse(url)))
        }
    }

    private fun cdmSection(user: DbxUser?) {
        button("CDM migration") {
            user ?. let {
                // This is not quite a no-op but queues a migration from the current
                // pathRoot to the same pathRoot
                it.accountManagerData.pendingPathRoot = it.accountManagerData.pendingPathRoot
            }
            startActivityForResult(
                CompanyDropboxMigrationActivity.getLaunchIntent(this),
                BaseActivityHelper.COMPANY_DROPBOX_MIGRATION_REQUEST_CODE
            )
        }
    }

    private fun deviceInfoSection() {
        text("Version", true)
        text(AppInfoUtils.getFullVersionString(this))

        mParent.addView(createDivider())
        text("Device IDs:", true)
        val envInfo = DropboxApplication.getEnvInfo(this)

        text("EnvInfo.getDeviceId: " + envInfo.deviceId, false, false)
        text("EnvInfo.getDeviceUDID: " + envInfo.deviceUDID, false, false)
        text("SystemInfo.getDeviceId: " + SystemInfo.getDeviceId(this), false, false)
    }

    /**
     * Gets an intent for a legacy shortcut created before the existence of multi-account. The
     * intent launched will have a URI that would only contain the path to a folder but no user id.
     *
     * This is used to test `MainBrowserLoadingFragment` as this is one of the few ways to access
     * that fragment.
     */
    private fun startLegacyShortcutIntent() {
        val intent = Intent("com.dropbox.BROWSE")
                .setComponent(
                        ComponentName(
                                "com.dropbox.android",
                                "com.dropbox.android.activity.DropboxBrowser"))
                .addCategory(Intent.CATEGORY_DEFAULT)
                .setData(
                        Uri.parse("content://com.dropbox.android.Dropbox/metadata/foobar"))

        startActivity(intent)
    }

    private fun createDivider(): View {
        val v = View(this)
        val lp = LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, 1)
        lp.gravity = Gravity.CENTER
        lp.topMargin = 10
        lp.bottomMargin = 10

        v.setBackgroundColor(Color.BLACK)
        v.layoutParams = lp
        return v
    }

    private fun text(text: String, bold: Boolean = false, center: Boolean = true) {
        mParent.addView(createText(text, bold, center))
    }

    private fun createText(text: String, bold: Boolean = false, center: Boolean = true): View {
        val t = TextView(this)
        val lp = LinearLayout.LayoutParams(
                LinearLayout.LayoutParams.WRAP_CONTENT,
                LinearLayout.LayoutParams.WRAP_CONTENT)
        lp.gravity = if (center) Gravity.CENTER else Gravity.LEFT

        t.text = text
        t.layoutParams = lp
        if (bold) {
            t.setTypeface(null, Typeface.BOLD)
        }
        return t
    }

    private fun button(text: String, onClick: () -> Unit) {
        mParent.addView(createButton(text, onClick))
    }

    fun DbxUser.button(text: String, onClick: (DbxUser) -> Unit) {
        button(text) { onClick(this) }
    }

    private fun createButton(text: String, onClick: () -> Unit): View {
        val b = Button(this)
        val lp = LinearLayout.LayoutParams(
                LinearLayout.LayoutParams.WRAP_CONTENT,
                LinearLayout.LayoutParams.WRAP_CONTENT)
        lp.gravity = Gravity.CENTER

        b.text = text
        b.setOnClickListener { onClick() }

        b.layoutParams = lp
        return b
    }

    private fun createSpinner(choices: List<String>, defaultPosition: Int): Spinner {
        val lp = LinearLayout.LayoutParams(
                LinearLayout.LayoutParams.WRAP_CONTENT,
                LinearLayout.LayoutParams.WRAP_CONTENT)
        lp.gravity = Gravity.CENTER

        val spinner = Spinner(this)
        spinner.layoutParams = lp
        val spinnerArrayAdapter = ArrayAdapter(this, android.R.layout.simple_spinner_item, choices)
        spinnerArrayAdapter.setDropDownViewResource(android.R.layout.simple_dropdown_item_1line)
        spinner.adapter = spinnerArrayAdapter
        spinner.setSelection(defaultPosition)
        return spinner
    }

    private fun createCheckbox(text: String, onCheck: () -> Unit, checked: Boolean): View {
        val checkBox = CheckBox(this)
        val layoutParams = LayoutParams(
                ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT)
        layoutParams.gravity = Gravity.CENTER
        checkBox.text = text
        checkBox.isChecked = checked
        checkBox.setOnCheckedChangeListener { _, _ -> onCheck() }
        checkBox.layoutParams = layoutParams
        return checkBox
    }

    private fun createScrollingContainer(): ViewGroup {
        val scroll = ScrollView(this)

        val mainLayout = LinearLayout(this)
        val mlp = FrameLayout.LayoutParams(
                ViewGroup.LayoutParams.MATCH_PARENT, FrameLayout.LayoutParams.WRAP_CONTENT)

        mainLayout.orientation = LinearLayout.VERTICAL
        scroll.addView(mainLayout, mlp)

        val slp = FrameLayout.LayoutParams(
                ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT)
        setContentView(scroll, slp)

        return mainLayout
    }
}
