/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */

package com.dropbox.android.filemanager

class GetLinkResult(
    val status: Status,
    val chooserResult: DbxChooserResult?,
    val userError: Int?
) {
    /**
     * The type of the result
     */
    enum class Status {
        /**
         * The get link operation completed successfully
         */
        SUCCESS,

        /**
         * The get link operation failed due to a network-related error
         */
        FAILED_NETWORK_ERROR,

        /**
         * The get link operation failed for an unknown reason
         */
        FAILED_UNKNOWN
    }

    companion object {
        /**
         * Constructs a failed result with no link created but with an optional userError
         * @param status The type of failure
         * @param userError Provided user visible error string, or `null`
         */
        fun forFailure(status: Status, userError: Int?): GetLinkResult {
            return GetLinkResult(
                    status, null, userError)
        }

        /**
         * Construct a success result with metadata to selected link
         */
        fun forSuccess(chooserResult: DbxChooserResult): GetLinkResult =
                GetLinkResult(Status.SUCCESS, chooserResult, null)
    }
}
