// dbx-min-coverage: 72% because dagger module
/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */

package com.dropbox.android.inject

import android.app.Application
import com.dropbox.android.exception.EnvInfo
import com.dropbox.android.util.LocaleUtils
import com.dropbox.android.util.SafeConsts
import com.dropbox.base.build_configuration.DevBuild
import com.dropbox.base.device.annotations.AppVersion
import com.dropbox.base.device.annotations.ServerTier
import com.dropbox.base.devsettings.DevSettings
import com.dropbox.base.devsettings.DevSettingsModule
import com.dropbox.base.http.AppKeyPair
import com.dropbox.core.service.DbxHttpHeaders
import dagger.Module
import dagger.Provides
import org.apache.commons.lang3.StringUtils
import javax.annotation.CheckForNull
import javax.annotation.Nullable
import javax.inject.Singleton

@Module(includes = [DevSettingsModule::class])
object DropboxApplicationModule {
    @Provides
    @Singleton
    @AppVersion
    @JvmStatic
    fun providesAppVersion(envInfo: EnvInfo, @Nullable devSettings: DevSettings?): String {
        return if (DevBuild.IS_DEV_BUILD &&
            !StringUtils.isEmpty(devSettings?.appVersionRequestHeader)
        ) {
            checkNotNull(devSettings).appVersionRequestHeader
        } else envInfo.appVersion
    }

    @Singleton
    @Provides
    @JvmStatic
    fun providesDbxHttpHeaders(
        @AppVersion appVersion: String,
        app: Application,
        @CheckForNull @ServerTier serverTier: String?
    ): DbxHttpHeaders =
        DbxHttpHeaders(appVersion, LocaleUtils.getLocaleForFormat(app.resources), serverTier)

    @Singleton
    @Provides
    @JvmStatic
    fun providesAppKeyPair(@Nullable devSettings: DevSettings?): AppKeyPair {
        if (DevBuild.IS_DEV_BUILD && devSettings != null && !devSettings.isProductionConfig) {
            return AppKeyPair(devSettings.consumerKey!!, devSettings.consumerSecret!!)
        }
        return AppKeyPair(SafeConsts.APP_KEY.toString(), SafeConsts.APP_SECRET.toString())
    }
}
