// dbx-min-coverage: 0% because wiring
/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */
@file:JvmName("CommentsWiring")

package com.dropbox.android.inject.comments

import com.dropbox.android.previewable.DevicePreviewableManager
import com.dropbox.android.user.DbxUserManager
import com.dropbox.core.v2.comments2.DbxUserComments2Requests
import com.dropbox.product.android.dbapp.comments.repository.CommentsMediaVerifier
import com.dropbox.product.android.dbapp.comments.repository.CommentsService
import com.dropbox.product.android.dbapp.comments.repository.CommentsServiceImpl
import com.dropbox.product.android.dbapp.comments.repository.CommentsWebServiceProvider
import com.dropbox.product.dbapp.path.DropboxPath
import com.dropbox.product.dbapp.path.Path
import com.dropbox.product.dbapp.path.SharedLinkPath

/**
 * Builds the service for use with [DropboxPath]s
 * @param userManager the [DbxUserManager] to use for providing the [DbxUserComments2Requests]
 * @param previewableManager the [DevicePreviewableManager] to use for media verification
 */
fun buildDropboxCommentService(
    userManager: DbxUserManager,
    previewableManager: DevicePreviewableManager
): CommentsService<DropboxPath> =
        CommentsServiceImpl(
                buildWebServiceProvider(userManager),
                buildMediaVerifier(previewableManager))

/**
 * Builds the service for use with [SharedLinkPath]s
 * @param userManager the [DbxUserManager] to use for providing the [DbxUserComments2Requests]
 * @param previewableManager the [DevicePreviewableManager] to use for media verification
 */
fun buildSharedLinkCommentService(
    userManager: DbxUserManager,
    previewableManager: DevicePreviewableManager
): CommentsService<SharedLinkPath> =
        CommentsServiceImpl(
                buildWebServiceProvider(userManager),
                buildMediaVerifier(previewableManager))

private fun buildMediaVerifier(previewableManager: DevicePreviewableManager) =
        object : CommentsMediaVerifier {
            override fun isVideoFile(path: Path): Boolean =
                previewableManager.isVideoFile(path.canonicalName)

            override fun isStreamingVideoFile(path: Path): Boolean =
                previewableManager.isStreamingVideoFile(path.canonicalName)

            override fun isAudioFile(path: Path): Boolean =
                previewableManager.isAudioFile(path.canonicalName)
        }

private fun buildWebServiceProvider(userManager: DbxUserManager) =
        object : CommentsWebServiceProvider {
            override fun commentsApiFor(userId: String?): DbxUserComments2Requests? {
                val users = userManager.users

                return users?.getUserById(userId)?.apiV2?.comments2()
            }
        }
