// dbx-min-coverage: 0% because dialog
package com.dropbox.android.openwith.ui

import android.app.Dialog
import android.content.Intent
import android.os.Bundle
import android.support.v4.app.FragmentActivity
import com.dropbox.android.R
import com.dropbox.android.activity.base.BasePathDialogFragment
import com.dropbox.android.asynctask.DownloadAndLaunchFileAsyncTask
import com.dropbox.android.util.PathHelper
import com.dropbox.core.ui.util.DbxAlertDialogBuilder
import com.dropbox.hairball.entry.LocalEntry
import com.dropbox.core.crash_reporting.CrashLogger
import com.dropbox.product.dbapp.path.Path

/**
 * Display an alert dialog warning the user that changes to the file made in a third-party app will
 * not be preserved because writing for the given app has been blocked.
 *
 * @param <P> The type of Path.
</P> */
class WriteBlockedAlertDialogFragment<P : Path> : BasePathDialogFragment<P>() {

    override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {
        val fragmentActivity = requireActivity()
        val entry = checkNotNull(arguments).getParcelable<LocalEntry<P>>(ARG_ENTRY)
        val appIntent = checkNotNull(arguments).getParcelable<Intent>(ARG_APP_LAUNCH_INTENT)

        return DbxAlertDialogBuilder(fragmentActivity).apply {
            setTitle(getString(R.string.open_with_read_only_application))
            setMessage(R.string.rosf_intent_warning_message)
            setNegativeButton(R.string.cancel, null)
            setPositiveButton(R.string.rosf_intent_warning_yes) { _, _ ->

                val task = DownloadAndLaunchFileAsyncTask.forIntent<FragmentActivity, P>(
                        activity,
                        entry,
                        pathHelper,
                        safeIntentStarter,
                        CrashLogger.instance,
                        appIntent)

                task.executeWithProgressDialog(
                        context, supportFragmentManager)
            }
        }.create()
    }

    companion object {

        private const val ARG_ENTRY = "ARG_ENTRY"
        private const val ARG_APP_LAUNCH_INTENT = "ARG_APP_LAUNCH_INTENT"

        /**
         * [WriteBlockedAlertDialogFragment] factory method
         *
         * @param entry Local entry
         * @param appIntent App intent that can be used to open this entry.
         *
         * @return instantiated WriteBlockedAlertDialogFragment
         */
        @JvmStatic
        fun <P : Path> newInstance(
            entry: LocalEntry<P>,
            pathHelper: PathHelper<P>,
            appIntent: Intent
        ) = WriteBlockedAlertDialogFragment<P>().apply {
            with(checkNotNull(arguments)) {
                putParcelable(ARG_ENTRY, entry)
                putParcelable(ARG_APP_LAUNCH_INTENT, appIntent)
                pathHelper.put(this)
            }
        }
    }
}
