// dbx-min-coverage: 0% because activity
/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */

package com.dropbox.android.preference

import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.support.v4.content.ContextCompat
import android.text.SpannableStringBuilder
import android.text.method.LinkMovementMethod
import com.dropbox.android.DropboxApplication
import com.dropbox.android.R
import com.dropbox.android.activity.base.BaseActivity
import com.dropbox.android.payments.PaymentUtils
import com.dropbox.android.util.TextWithLinks
import com.dropbox.base.analytics.AnalyticsLogger
import com.dropbox.base.analytics.ManageYourSubscriptionEvents
import kotlinx.android.synthetic.main.manage_subscription_activity.*

/**
 * Screen that re-iterates the values of Dropbox Plus and allows the users to access instructions on
 * how to unsubscribe from Dropbox Plus.
 */
class ManageSubscriptionActivity : BaseActivity() {

    companion object {

        @JvmStatic
        fun getLaunchIntent(context: Context) = Intent(
            context,
            ManageSubscriptionActivity::class.java
        )
    }

    private lateinit var analyticsLogger: AnalyticsLogger

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        analyticsLogger = DropboxApplication.getDefaultEventLogger(this)

        setContentView(R.layout.manage_subscription_activity)

        setSupportActionBar(toolbarLayout.toolbar)
        checkNotNull(supportActionBar).setDisplayHomeAsUpEnabled(true)
        setTitle(R.string.manage_subscription_title)

        val checkMarkSize =
            resources.getDimensionPixelSize(R.dimen.manage_subscription_check_mark_size)
        val checkMarkDrawable = checkNotNull(
            ContextCompat.getDrawable(this, R.drawable.ic_done_blue_24dp)
        ).apply {
            setBounds(0, 0, checkMarkSize, checkMarkSize)
        }

        listOf(bullet1TextView, bullet2TextView, bullet3TextView, bullet4TextView).forEach {
            it.setCompoundDrawables(checkMarkDrawable, null, null, null)
            it.compoundDrawablePadding =
                    resources.getDimensionPixelSize(R.dimen.manage_subscription_check_mark_padding)
        }

        changeOrCancelPlanButton.setOnClickListener { showManageSubscriptionHelpArticle() }

        val textWithLinks = TextWithLinks(getString(R.string.manage_subscription_sub_text))
        // There should be exactly one link in the text.
        check(textWithLinks.linkSpans.size == 1)

        subTextView.apply {
            movementMethod = LinkMovementMethod.getInstance()
            text = SpannableStringBuilder(textWithLinks.toString()).apply {
                val link = textWithLinks.linkSpans[0]
                TextWithLinks.makeLink(
                    resources,
                    this,
                    link.first,
                    link.second,
                    ::showManageSubscriptionHelpArticle
                )
            }
        }
    }

    private fun showManageSubscriptionHelpArticle() {
        ManageYourSubscriptionEvents.TapChangeOrCancelPlan().logTo(analyticsLogger)
        startActivity(PaymentUtils.getIntentForManageSubscriptionHelpArticle(resources))
    }
}
