// dbx-min-coverage: 0% because Preference
/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */

package com.dropbox.android.preference

import android.content.Context
import android.support.v7.preference.Preference
import android.support.v7.preference.PreferenceViewHolder
import android.util.AttributeSet
import android.widget.TextView
import com.dropbox.android.R

/**
 * A preference with a widget that can display on or off.
 */
class OnOffPreference : Preference {

    var isOn = false

    constructor(context: Context?) : super(context)

    constructor(context: Context?, attrs: AttributeSet?) : super(context, attrs)

    constructor(
        context: Context?,
        attrs: AttributeSet?,
        defStyleAttr: Int
    ) : super(context, attrs, defStyleAttr)

    constructor(
        context: Context?,
        attrs: AttributeSet?,
        defStyleAttr: Int,
        defStyleRes: Int
    ) : super(context, attrs, defStyleAttr, defStyleRes)

    init {
        widgetLayoutResource = R.layout.dbx_preference_widget_on_off_text
    }

    override fun onBindViewHolder(holder: PreferenceViewHolder?) {
        super.onBindViewHolder(holder)

        holder ?: return

        val onOffTextView = holder.findViewById(R.id.on_off_text_view) as TextView

        if (isOn) {
            onOffTextView.setText(R.string.settings_on)
        } else {
            onOffTextView.setText(R.string.settings_off)
        }
    }

    fun setOn() {
        isOn = true
        notifyChanged()
    }

    fun setOff() {
        isOn = false
        notifyChanged()
    }
}
