// dbx-min-coverage: 0% because Preference
/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */

package com.dropbox.android.preference

import android.content.Context
import android.support.v7.preference.Preference
import android.support.v7.preference.PreferenceViewHolder
import android.util.AttributeSet
import android.view.View
import android.widget.Button
import com.dropbox.android.R

/**
 * A preference that promotes dropbox plus. Variant 2.
 */
class UserUpsellV2Preference : Preference {

    private var positiveButtonClickListener: View.OnClickListener? = null
    private var negativeButtonClickListener: View.OnClickListener? = null

    constructor(context: Context?) : super(context)

    constructor(context: Context?, attrs: AttributeSet?) : super(context, attrs)

    constructor(
        context: Context?,
        attrs: AttributeSet?,
        defStyleAttr: Int
    ) : super(context, attrs, defStyleAttr)

    constructor(
        context: Context?,
        attrs: AttributeSet?,
        defStyleAttr: Int,
        defStyleRes: Int
    ) : super(context, attrs, defStyleAttr, defStyleRes)

    init {
        layoutResource = R.layout.dbx_user_upsell_v2_preference
    }

    override fun onBindViewHolder(holder: PreferenceViewHolder?) {
        super.onBindViewHolder(holder)

        holder ?: return

        val positiveButton = holder.findViewById(R.id.positive_button) as Button
        val negativeButton = holder.findViewById(R.id.negative_button) as Button

        positiveButton.setOnClickListener(positiveButtonClickListener)
        negativeButton.setOnClickListener(negativeButtonClickListener)
    }

    fun setPositiveButtonOnClickListener(onClickListener: View.OnClickListener) {
        positiveButtonClickListener = onClickListener
        notifyChanged()
    }

    fun setNegativeButtonOnClickListener(onClickListener: View.OnClickListener) {
        negativeButtonClickListener = onClickListener
        notifyChanged()
    }
}
