/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */
package com.dropbox.android.sharing.link

import com.dropbox.hairball.entry.SharedLinkEntry

enum class SharedLinkSuccessAction {
    REDIRECT_TO_BROWSE_FOLDER,
    REDIRECT_TO_BROWSE_FILE,
    PREVIEW_FOLDER,
    MOUNT_OR_PREVIEW_FOLDER,
    PREVIEW_FILE;
}

/**
 * Parse a APIv1 shared link metadata entry to determine what action should
 * be taken on that shared link.
 */
internal fun SharedLinkEntry.getAsSuccessAction() =
    if (inDropbox) {
        if (isDir) {
            SharedLinkSuccessAction.REDIRECT_TO_BROWSE_FOLDER
        } else {
            SharedLinkSuccessAction.REDIRECT_TO_BROWSE_FILE
        }
    } else {
        if (isDir) {
            if (sharedContentLinkEntry != null && permissions.canSync) {
                SharedLinkSuccessAction.MOUNT_OR_PREVIEW_FOLDER
            } else {
                SharedLinkSuccessAction.PREVIEW_FOLDER
            }
        } else {
            SharedLinkSuccessAction.PREVIEW_FILE
        }
    }
