/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */
// dbx-min-coverage: 0% because these are helper functions converting between two different enum classes
@file:JvmName("ThumbnailConverter")
package com.dropbox.android.taskqueue

import com.dropbox.client2.DropboxAPI
import com.dropbox.core.v2.files.ThumbnailFormat
import com.dropbox.core.v2.files.ThumbnailMode
import com.dropbox.core.v2.files.ThumbnailSize

/**
 * These converter functions help map ApiV1 thumnail size/format to ApiV2's size/format during migration
 */

internal fun DropboxAPI.ThumbSize.convertToThumbnailSize(): ThumbnailSize {
    return when (this) {
        DropboxAPI.ThumbSize.ICON_128x128 -> ThumbnailSize.W128H128
        DropboxAPI.ThumbSize.ICON_256x256 -> ThumbnailSize.W256H256
        DropboxAPI.ThumbSize.BESTFIT_640x480 -> ThumbnailSize.W640H480
        DropboxAPI.ThumbSize.BESTFIT_960x640 -> ThumbnailSize.W960H640
        DropboxAPI.ThumbSize.BESTFIT_1024x768 -> ThumbnailSize.W1024H768
        DropboxAPI.ThumbSize.BESTFIT_FITONE_256 -> ThumbnailSize.W256H256
    }
}

internal fun DropboxAPI.ThumbSize.convertToThumbnailMode(): ThumbnailMode {
    return when (this) {
        DropboxAPI.ThumbSize.ICON_128x128 -> ThumbnailMode.STRICT
        DropboxAPI.ThumbSize.ICON_256x256 -> ThumbnailMode.STRICT
        DropboxAPI.ThumbSize.BESTFIT_640x480 -> ThumbnailMode.BESTFIT
        DropboxAPI.ThumbSize.BESTFIT_960x640 -> ThumbnailMode.BESTFIT
        DropboxAPI.ThumbSize.BESTFIT_1024x768 -> ThumbnailMode.BESTFIT
        DropboxAPI.ThumbSize.BESTFIT_FITONE_256 -> ThumbnailMode.FITONE_BESTFIT
    }
}

internal fun DropboxAPI.ThumbFormat.convertToThumbnailFormat(): ThumbnailFormat {
    return when (this) {
        DropboxAPI.ThumbFormat.PNG -> ThumbnailFormat.PNG
        DropboxAPI.ThumbFormat.JPEG -> ThumbnailFormat.JPEG
    }
}
