/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */

package com.dropbox.android.util

import com.dropbox.hairball.entry.CloudDocClass
import com.dropbox.hairball.entry.LocalEntry
import com.dropbox.product.dbapp.path.Path

/**
 * Whether or not a file is a cloud doc that exists outside the regular dropbox filesystem
 * (i.e. its file contents does not meaningfully represent the file itself) in a way that would
 * require us to disable device filesystem-based (e.g. export) actions.
 */
fun <T : Path> LocalEntry<T>.isCloudDocOutsideFilesystem(): Boolean =
    when (cloudDocClass) {
        CloudDocClass.CLOUD_BACKED_FILE -> false
        CloudDocClass.CANONICAL -> true
        CloudDocClass.OTHER -> {
            // This it used to represent a value that we received from the server that we couldn't
            // understand (i.e. a future type of cloud doc).
            // Because we don't know anything about this doc (such as whether it makes sense
            // to export such a thing), we should return true here with the aim of disabling
            // actions that should not be exposed to the user (e.g. export).
            true
        }
        CloudDocClass.NONE -> false
    }
